package fr.bhprod.sga.terrain;

import org.apache.log4j.Logger;

import com.jme.math.Ray;
import com.jme.math.Triangle;
import com.jme.math.Vector3f;

/**
 * 
 * @author jbeaufils
 * @version 1.0
 */

public final class TerrainTools {

	public static final Logger LOGGER = Logger.getLogger(TerrainTools.class);

	/**
	 * Return the position of the cross between a point over the terrain and the terrain.
	 * @param position The point
	 * @return The position and the normal vector
	 */
	public static TerrainPosition getTerrainPosition(final Terrain terrain, final Vector3f position) {
		Vector3f currentPosition = new Vector3f(position);
		float px = currentPosition.x / terrain.getTileRealSize();
		float pz = -currentPosition.z / terrain.getTileRealSize();

		int tilex = (int)px;
		int tilez = (int)pz;
		
		LOGGER.debug("px : "+px);
		LOGGER.debug("pz : "+pz);
		LOGGER.debug("tilex : "+tilex);
		LOGGER.debug("tilez : "+tilez);
		LOGGER.debug("position : "+currentPosition);
		
		if (px == tilex && pz == tilez) {
			LOGGER.info("L'objet est au coin d'un tile");
			return new TerrainPosition(new Vector3f(currentPosition.x, terrain.getHeights()[tilex][tilez], currentPosition.z), null);
		} else if (px == tilex) {
			LOGGER.info("L'objet est au bord d'un tile");
			currentPosition.x += 0.0001;
		} else if (pz == tilez) {
			LOGGER.info("L'objet est au bord d'un tile");
			currentPosition.z -= 0.0001;
		}
		
		Ray ray = new Ray(currentPosition,new Vector3f(0,-1,0));
		
		Triangle t0 = new Triangle(getCornerCoordinates(terrain, tilex,tilez),getCornerCoordinates(terrain, tilex+1,tilez),getCornerCoordinates(terrain, tilex,tilez+1));

		TerrainPosition res = getClickCoordinates(ray, t0);
		if (res != null) {
			LOGGER.debug("result : "+res);
			return res;
		}

		Triangle t1 = new Triangle(getCornerCoordinates(terrain, tilex+1,tilez),getCornerCoordinates(terrain, tilex+1,tilez+1),getCornerCoordinates(terrain, tilex,tilez+1));

		res = getClickCoordinates(ray, t1);
		if (res != null) {
			LOGGER.debug("result : "+res);
			return res;
		}

		LOGGER.debug("no result");
		return null;
	}

	/**
	 * Returns the coordinates of the front left corner of a tile.
	 * @param x X position of the corner
	 * @param z Z position of the corner
	 * @return The coordinates
	 */
	private static Vector3f getCornerCoordinates(final Terrain terrain, final int x, final int z) {
		Vector3f vec = new Vector3f();
		vec.x = x * terrain.getTileRealSize();
		vec.y = terrain.getHeights()[x][z];
		vec.z = z * - terrain.getTileRealSize();
		return vec;
	}

	/**
	 * Get the normal vector of a triangle.
	 *
	 * @param t The triangle
	 * @return normal of triangle, same as store param.
	 */
	private static Vector3f getTriangleNormal(final Triangle t) {
		Vector3f store = new Vector3f();
		
		Vector3f v1 = new Vector3f(t.get(1));
		Vector3f v2 = new Vector3f(t.get(2));
		
		v1.subtractLocal(t.get(0));
		v2.subtractLocal(t.get(0));
		v1.cross(v2, store);

		return store;
	}
	
	/**
	 * Returns the coordinates of the intersection between a ray and a triangle.
	 * @param ray The ray
	 * @param tileName The name of the tile object
	 * @return The intersection point or null if there is no intersection
	 */
	public static TerrainPosition getClickCoordinates(final Ray ray, final Triangle triangle) {
		Vector3f v = new Vector3f();
		
		if (ray.intersectWhere(triangle, v)) {
			return new TerrainPosition(v, getTriangleNormal(triangle));
		}

		return null;
	}
	
}

